/*                                                                             
 * Copyright 2004,2005 The Apache Software Foundation.                         
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.carbon.bridge;

import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Collections;
import java.util.concurrent.ConcurrentHashMap;

/**
 * This class is used for holding all the HttpSessions created in Carbon. It is used for
 * managing & invalidating all sessions created in the system
 */
public class HttpSessionManager {
    private static Map<String, CarbonHttpSession> sessions =
            new ConcurrentHashMap<String, CarbonHttpSession>();

    static {
        Timer timer = new Timer();
        timer.scheduleAtFixedRate(new SessionCleanupTask(), 0, 5000);
    }

    private static class SessionCleanupTask extends TimerTask {
        public void run() {
            try {
                for (HttpSession session : sessions.values()) {
                    try {
                        session.getLastAccessedTime();
                    } catch (IllegalStateException e) {
                        // This exception is thrown when getLastAccessedTime is called on an
                        // invalidated session. So we can safely remove it.
                        sessions.remove(session.getId());
                    }
                }
            } catch (Throwable e) {
                e.printStackTrace(); // Cannot log since there is no dependency to log4j from the bridge
            }
        }
    }

    public static void addSession(CarbonHttpSession session) {
        sessions.put(session.getId(), session);
    }

    public static CarbonHttpSession getSession(String sessionId) {
        return sessions.get(sessionId);
    }

    public static void invalidateSessions() {
        for (HttpSession session : sessions.values()) {
            try {
                session.invalidate();
            } catch (Throwable ignored) {
            }
        }
        sessions.clear();
    }
}
